@@ -35,6 +35,8 @@ from pyroute2.netlink.nfnetlink.ipset import IPSET_CMD_RENAME
 from pyroute2.netlink.nfnetlink.ipset import IPSET_CMD_TEST
 from pyroute2.netlink.nfnetlink.ipset import IPSET_CMD_TYPE
 from pyroute2.netlink.nfnetlink.ipset import IPSET_CMD_HEADER
+from pyroute2.netlink.nfnetlink.ipset import IPSET_CMD_GET_BYNAME
+from pyroute2.netlink.nfnetlink.ipset import IPSET_CMD_GET_BYINDEX
 from pyroute2.netlink.nfnetlink.ipset import ipset_msg
 from pyroute2.netlink.nfnetlink.ipset import IPSET_FLAG_WITH_COUNTERS
 from pyroute2.netlink.nfnetlink.ipset import IPSET_FLAG_WITH_COMMENT
@@ -103,7 +105,9 @@ class IPSet(NetlinkSocket):
     policy = {IPSET_CMD_PROTOCOL: ipset_msg,
               IPSET_CMD_LIST: ipset_msg,
               IPSET_CMD_TYPE: ipset_msg,
-              IPSET_CMD_HEADER: ipset_msg}
+              IPSET_CMD_HEADER: ipset_msg,
+              IPSET_CMD_GET_BYNAME: ipset_msg,
+              IPSET_CMD_GET_BYINDEX: ipset_msg}
 
     attr_map = {'iface': 'IPSET_ATTR_IFACE',
                 'mark': 'IPSET_ATTR_MARK',
@@ -117,14 +121,17 @@ class IPSet(NetlinkSocket):
                 ('ip_to', 1): 'IPSET_ATTR_IP_TO',
                 ('ip_to', 2): 'IPSET_ATTR_IP2_TO'}
 
-    def __init__(self, version=6, attr_revision=None, nfgen_family=2):
+    def __init__(self, version=None, attr_revision=None, nfgen_family=2):
         super(IPSet, self).__init__(family=NETLINK_NETFILTER)
         policy = dict([(x | (NFNL_SUBSYS_IPSET << 8), y)
                        for (x, y) in self.policy.items()])
         self.register_policy(policy)
+        self._nfgen_family = nfgen_family
+        if version is None:
+            msg = self.get_proto_version()
+            version = msg[0].get_attr('IPSET_ATTR_PROTOCOL')
         self._proto_version = version
         self._attr_revision = attr_revision
-        self._nfgen_family = nfgen_family
 
     def request(self, msg, msg_type,
                 msg_flags=NLM_F_REQUEST | NLM_F_DUMP,
@@ -137,14 +144,25 @@ class IPSet(NetlinkSocket):
         except NetlinkError as err:
             raise _IPSetError(err.code, cmd=msg_type)
 
-    def headers(self, name):
+    def headers(self, name, **kwargs):
         '''
         Get headers of the named ipset. It can be used to test if one ipset
         exists, since it returns a no such file or directory.
         '''
-        return self._list_or_headers(IPSET_CMD_HEADER, name=name)
+        return self._list_or_headers(IPSET_CMD_HEADER, name=name, **kwargs)
 
-    def list(self, *argv, **kwarg):
+    def get_proto_version(self, version=6):
+        '''
+        Get supported protocol version by kernel.
+
+        version parameter allow to set mandatory (but unused?)
+        IPSET_ATTR_PROTOCOL netlink attribute in the request.
+        '''
+        msg = ipset_msg()
+        msg['attrs'] = [['IPSET_ATTR_PROTOCOL', version]]
+        return self.request(msg, IPSET_CMD_PROTOCOL)
+
+    def list(self, *argv, **kwargs):
         '''
         List installed ipsets. If `name` is provided, list
         the named ipset or return an empty list.
@@ -153,14 +171,16 @@ class IPSet(NetlinkSocket):
         exit, you will receive an empty list.
         '''
         if len(argv):
-            kwarg['name'] = argv[0]
-        return self._list_or_headers(IPSET_CMD_LIST, **kwarg)
+            kwargs['name'] = argv[0]
+        return self._list_or_headers(IPSET_CMD_LIST, **kwargs)
 
-    def _list_or_headers(self, cmd, name=None):
+    def _list_or_headers(self, cmd, name=None, flags=None):
         msg = ipset_msg()
         msg['attrs'] = [['IPSET_ATTR_PROTOCOL', self._proto_version]]
         if name is not None:
             msg['attrs'].append(['IPSET_ATTR_SETNAME', name])
+        if flags is not None:
+            msg['attrs'].append(['IPSET_ATTR_FLAGS', flags])
         return self.request(msg, cmd)
 
     def destroy(self, name=None):
@@ -446,6 +466,35 @@ class IPSet(NetlinkSocket):
                             msg_flags=NLM_F_REQUEST | NLM_F_ACK,
                             terminate=_nlmsg_error)
 
+    def _get_set_by(self, cmd, value):
+        # Check that IPSet version is supported
+        if self._proto_version < 7:
+            raise NotImplementedError()
+
+        msg = ipset_msg()
+        if cmd == IPSET_CMD_GET_BYNAME:
+            msg['attrs'] = [['IPSET_ATTR_PROTOCOL', self._proto_version],
+                            ['IPSET_ATTR_SETNAME', value]]
+
+        if cmd == IPSET_CMD_GET_BYINDEX:
+            msg['attrs'] = [['IPSET_ATTR_PROTOCOL', self._proto_version],
+                            ['IPSET_ATTR_INDEX', value]]
+        return self.request(msg, cmd)
+
+    def get_set_byname(self, name):
+        '''
+        Get a set by its name
+        '''
+
+        return self._get_set_by(IPSET_CMD_GET_BYNAME, name)
+
+    def get_set_byindex(self, index):
+        '''
+        Get a set by its index
+        '''
+
+        return self._get_set_by(IPSET_CMD_GET_BYINDEX, index)
+
     def get_supported_revisions(self, stype, family=socket.AF_INET):
         '''
         Return minimum and maximum of revisions supported by the kernel.
